/*
 * ditoh@andrew.cmu.edu
 * MTI Spring '18
 * Assignment 4
 * 
 * Goal is to visualize the drum beat strength to assist the player
 * to play drum effectively. Using Arduino and piezo sensor to detect
 * magnitude of hit and P5JS for visualization.
 */


// pin variables
static const int vibPin = A0;

// piezo variables
// set sample to 1 since vibration value is a high frequency spike
static const int vibSize = 1;
unsigned int vibReads[vibSize];
unsigned int vibIndex = 0;

// variables for knock read and record
static const int knockSize = 20;
unsigned int knockReads[knockSize];
unsigned int knockIndex = 0;
unsigned long startMillis = 0;
unsigned int threshMinMillis = 250; // ignore multiple spikes in single knock


void setup() {
  
  // serial setting
  Serial.begin(9600);
  
  // pin setting
  pinMode(vibPin, INPUT);
}


void loop() {  
  
  // read sensor inputs every loop
  vibReads[vibIndex] = analogRead(vibPin);

  // piezo input processing
  if (vibIndex < (vibSize - 1)) {
    vibIndex++;
  }
  else {
    unsigned int mean = computeStat(vibReads, vibSize);
    if (knockIndex > 0 && millis() - startMillis > threshMinMillis) {
      knockIndex = 0;
      startMillis = 0;
    }
    if (knockIndex == 0 && mean > 30) {
      startMillis = millis();
      knockIndex++;
      mean = map(mean, 0, 150, 0, 255);
      Serial.write(mean);
      //Serial.println(mean);
    }
    
    // reset
    vibIndex = 0;
  }
}


unsigned int computeStat(unsigned int readings[], unsigned int sampleSize) {
  
  unsigned int sum = 0;
  unsigned int low = 1024;
  unsigned int high = 0;

  // loop samples
  for (int i = 0; i < sampleSize; i++) {
    // ignore outliers
    if (readings[i] < 1024) {
      sum += readings[i];
      
      if (readings[i] < low) {
        low = readings[i];
      }
      if (readings[i] > high) {
        high = readings[i];
      }
    }
  }

  return sum / sampleSize;
}

